
/* dranduniform_secrng Example Program Text */
/*
 * Copyright (C) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
 */

#include <rng.h>
#include <stdio.h>
#include <stdlib.h>


#define MN   12

rng_int_t *state = NULL;
double    x[MN];


int main(void)
{
  const int mn = MN;
  double a, b;
  rng_int_t n, genid, subid, lstate, info;


  printf("\n");
  printf("--RNG-example: uniform random numbers using dranduniform_secrng--\n");
  printf("----------------------------------------------------------\n");
  printf("\n");

  /* Initialize the number of variates required */
  n = mn;

  /* Use the MT generator as the base generator */
  genid = 3;
  subid = 1;

  lstate = 0;

  /* Initialize the base generator */
  drandinitialize_secrng(genid, subid, state, &lstate, &info);

  /* Allocate memory for state */
  state = (rng_int_t *)malloc(lstate * sizeof(rng_int_t));
  if (!state) {
    printf("Error, could not allocate memory\n");
    exit(1);
  }

 /* Initialize the base generator */
  drandinitialize_secrng(genid, subid, state, &lstate, &info);
  if (info < 0) {
    printf("Error, initialization is not successful !!!\n");
    printf("Sufficient RN# are not generated by HW  !!!\n");
    free(state);
    exit(0);
  }

  /* Generate a sequence from a uniform U(0,1) distribution */
  a = 0.0;
  b = 1.0;

  dranduniform(n, a, b, state, x, &info);

  /* Print the sequence */
  printf("Numbers from a uniform U(0,1) distribution:\n");

  for (int i=0; i<n; i++) {
    printf("%10.4f", x[i]);

    if ((i+1)%4 == 0)
      printf("\n");
  }

  printf("\n");

  free(state);
  return 0;
}
